package com.rshy.project.hy.manager.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.rshy.project.hy.manager.RshyJkManager;
import com.rshy.project.hy.model.enums.HyTypeEnum;
import com.rshy.project.hy.model.enums.PassStatusEnum;
import com.rshy.project.hy.model.enums.RepositoryEnum;
import com.rshy.project.hy.model.enums.SexEnum;
import com.rshy.project.hy.server.FaceRetrievaServer;
import com.rshy.project.hy.server.ItfwServer;
import com.rshy.project.hy.server.dto.PctjkmInfoDTO;
import com.rshy.project.hy.server.dto.RepositoryDTO;
import com.rshy.project.hy.server.dto.ResultDTO;
import com.rshy.project.hy.server.param.RshyParam;
import com.rshy.project.hy.server.param.RshyQuickParam;
import com.rshy.project.hy.server.vo.RetrievaResultVO;
import com.rshy.project.hy.server.vo.RetrievalRepositoryVO;
import com.rshy.project.hy.server.vo.RshyVo;
import com.rshy.project.hy.util.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/23 20:12
 * @description:
 */
@Component
@Slf4j
public class RshyJkManagerImpl implements RshyJkManager {
    @Autowired
    private FaceRetrievaServer faceRetrievaServer;
    @Autowired
    private ItfwServer itfwServer;

    @Override
    public RshyVo getRshyInfo(RshyParam rshyParam) {
        log.info("普通核验接口请求参数：{}", JSON.toJSONString(rshyParam));
        RshyVo rshyVo = new RshyVo();
        if (StringUtils.isNotBlank(rshyParam.getBarCode())) {
            PctjkmInfoDTO barCodes = null;
            try {
                //查询健康码信息
                 barCodes = itfwServer.getPctjkm(rshyParam.getBarCode(), rshyParam.getTimeout());
            }catch (Exception e){
                log.error("查询健康码信息异常:{}", e);
            }

            if (barCodes == null) {
                return rshyVo;
            }

            rshyParam.setSfzh(barCodes.getId_num());
            rshyParam.setXm(barCodes.getUser_name());
        }

        ResultDTO resultDTO = new ResultDTO();
        List<Runnable> taskList = new ArrayList<Runnable>() {
            {
                add(() -> resultDTO.setPctdfkDTO(itfwServer.getPctdfk(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
                add(() -> resultDTO.setPctjkmInfoDTO(itfwServer.getPctjkm(rshyParam.getBarCode(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctjsryDTO(itfwServer.getPctjsry(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctztDTO(itfwServer.getPctzt(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctzfbaDTO(itfwServer.getPctzfba(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctqkryDTO(itfwServer.getPctqkry(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctxdryDTO(itfwServer.getPctxdry(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
                add(() -> resultDTO.setPctzdryDTO(itfwServer.getPctzdry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            }
        };

        //异步流并发处理
        taskList.parallelStream().forEach(v -> {
            try {
                v.run();
            } catch (Exception e) {
                log.error("异常信息：{}", e);
            }
        });

        String codeLabel = resultDTO.getPctjkmInfoDTO() != null ? resultDTO.getPctjkmInfoDTO().getCOLOR_CODE_LABEL() : null;
        String phone = resultDTO.getPctjkmInfoDTO() != null ? resultDTO.getPctjkmInfoDTO().getPn() : null;
        rshyVo.setXm(rshyParam.getXm()).setSfzh(rshyParam.getSfzh()).setHealthCode(codeLabel).setPhone(phone);
        resultDTO.setPctjkmInfoDTO(null);

        if (resultDTO.getPctztDTO() != null) {
            rshyVo.setPass(PassStatusEnum.NO_PASSING.getCode()).setTts(PassStatusEnum.NO_PASSING.getDesc()).setDisplayMsg(PassStatusEnum.NO_PASSING.getDesc()).setWarning(new StringBuilder().append("在逃人员:").append(resultDTO.getPctztDTO().getJyaq()).toString()).setKind(resultDTO.getPctztDTO().getZtrylxmc());
        } else if (ObjectUtils.allFieldIsNULL(resultDTO)) {
            rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode()).setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc()).setTts(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
        } else {
            rshyVo.setKind(kind(resultDTO)).setWarning(warning(resultDTO)).setDisplayMsg(PassStatusEnum.RIGHT_WAY.getDesc()).setTts(PassStatusEnum.RIGHT_WAY.getDesc());
        }

        log.info("普通核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }


    @Override
    public RshyVo getQuickRyhyInfo(RshyQuickParam rshyQuickParam) {
        RshyVo rshyVo = new RshyVo();
        RetrievaResultVO retrievaResultVO = faceRetrievaServer.retrieval(rshyQuickParam.getPhoto(), rshyQuickParam.getTimeout());
        List<RetrievalRepositoryVO> voList = Convert.toList(RetrievalRepositoryVO.class, retrievaResultVO.getResults());
        if (CollectionUtil.isEmpty(voList)) {
            rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode());
            rshyVo.setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
            rshyVo.setTts(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
            return rshyVo;
        }

        //查询人像库信息
        RetrievaResultVO repository = faceRetrievaServer.repository(rshyQuickParam.getTimeout());
        List<RepositoryDTO> dtoList = Convert.toList(RepositoryDTO.class, repository.getResults());
        Map<String, String> stringMap = CollectionUtil.isNotEmpty(dtoList) ? dtoList.stream().collect(Collectors.toMap(RepositoryDTO::getId, RepositoryDTO::getName, (v1, v2) -> v1)) : MapUtil.newHashMap();
        RetrievalRepositoryVO repositoryVO = voList.stream().findFirst().get();
        rshyVo.setPass(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getCode() : PassStatusEnum.RIGHT_WAY.getCode()).setKind(StrUtil.EMPTY)
                .setWarning(stringMap.get(repositoryVO.getRepository_id())).setDisplayMsg(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc())
                .setTts(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc()).setHealthCode(StrUtil.EMPTY).setSfzh(repositoryVO.getPersonId())
                .setXm(repositoryVO.getName()).setXb(SexEnum.codeOf(repositoryVO.getGender()).getDesc()).setMz(StrUtil.EMPTY).setPhone(StrUtil.EMPTY).setAddress(StrUtil.EMPTY).setPhoto(rshyQuickParam.getPhoto());
        log.info("快速核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }

    /**
     * 人员类型简要概述
     *
     * @param resultDTO
     * @return
     */
    private String kind(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctdfkDTO() != null ? HyTypeEnum.pctdfk.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjsryDTO() != null ? HyTypeEnum.pctjsry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctxdryDTO() != null ? HyTypeEnum.pctxdry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctqkryDTO() != null ? HyTypeEnum.pctqkry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjkmInfoDTO() != null ? HyTypeEnum.pctjkm.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzfbaDTO() != null ? HyTypeEnum.pctzfba.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzdryDTO() != null ? HyTypeEnum.pctzdry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctztDTO() != null ? HyTypeEnum.pctzt.getDesc() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }

    /**
     * 警示信息
     *
     * @param resultDTO
     * @return
     */
    private String warning(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctdfkDTO() != null ? resultDTO.getPctdfkDTO().getJyaq() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjsryDTO() != null ? resultDTO.getPctjsryDTO().getJyaq() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctxdryDTO() != null ? resultDTO.getPctxdryDTO().getXdcsmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctqkryDTO() != null ? resultDTO.getPctqkryDTO().getLsqkqk() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzfbaDTO() != null ? resultDTO.getPctzfbaDTO().getAjmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzdryDTO() != null ? resultDTO.getPctzdryDTO().getAjlbmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctztDTO() != null ? resultDTO.getPctztDTO().getJyaq() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }
}
